/* =============================================================================
 * conjunto.c
 *
 * Estructura de datos y primitivas del TAD Conjunto - Versin 1.
 *
 * En esta versin los elementos de un conjunto son int.
 * Por ello, algunas de las primitivas del TAD son dependientes de ese tipo de
 * dato.
 * Esta implementacin del TAD no vale para almacenar otro tipo de dato.
 * Si se quiere tener un conjunto con elementos que no sean int, habra que
 * implementar un nuevo TAD.
 *
 * Autor: Ivn Cantador
 * Fecha: 9/2/2017
 * Versin: 1.0
 * Comentarios: En este fichero habra que incluir las cabeceras de funcin,
 * describiendo las primitivas, sus argumentos de entrada y valores de salida.
 * ========================================================================== */

#include <stdio.h>
#include <stdlib.h>
#include "tipos.h"
#include "conjunto.h"

/* -------------------------- Estructura de datos -------------------------- */
/* ------------------------------------------------------------------------- */

#define MAX_ELEMENTOS 1000

struct _Conjunto {
	/* Un elemento es un int */
	int elementos[MAX_ELEMENTOS];
	int numElementos;
};

/* -------------------------- Funciones primitivas -------------------------- */
/* -------------------------------------------------------------------------- */

Conjunto *conjunto_crear() {
	Conjunto *cjto = NULL;

	cjto = (Conjunto *) malloc(sizeof(Conjunto));
	if ( cjto == NULL ) {
		return NULL;
	}

	cjto->numElementos = 0;

	return cjto;
}

void conjunto_vaciar(Conjunto *cjto) {
	if ( cjto != NULL ) {
		cjto->numElementos = 0;
	}
}

void conjunto_eliminar(Conjunto *cjto) {
	if ( cjto != NULL ) {
		free(cjto);
	}
}

int conjunto_cardinalidad(Conjunto *cjto) {
	if ( cjto == NULL ) {
		return -1;
	}

	return cjto->numElementos;
}

boolean conjunto_estaVacio(Conjunto *cjto) {
	if ( cjto == NULL ) {
		return NULL_BOOLEAN;
	}

	return cjto->numElementos == 0 ? TRUE : FALSE;
}

boolean conjunto_estaLleno(Conjunto *cjto) {
	if ( cjto == NULL ) {
		return NULL_BOOLEAN;
	}

	return cjto->numElementos == MAX_ELEMENTOS ? TRUE : FALSE;
}

int conjunto_obtenerElemento(Conjunto *cjto, int pos) {
	if ( cjto == NULL || pos < 0 || pos >= cjto->numElementos ) {
		return -1;
	}

	return cjto->elementos[pos];
}

int conjunto_buscarElemento(Conjunto *cjto, int elem) {
	int i;

	if ( cjto == NULL ) {
		return NAN;
	}

	for ( i = 0 ; i < cjto->numElementos ; i++ ) {
		if ( cjto->elementos[i] == elem ) {
			return i;
		}
	}

	return -1;
}

status conjunto_insertarElemento(Conjunto *cjto, int elem) {
	if ( cjto == NULL || cjto->numElementos == MAX_ELEMENTOS ) {
		return ERROR;
	}

	if ( conjunto_buscarElemento(cjto, elem) != -1 ) {
		return OK;
	}

	cjto->elementos[cjto->numElementos] = elem;
	cjto->numElementos++;

	return OK;
}

void conjunto_visualizar(Conjunto *c) {
	int i, n, e;

	if ( c != NULL ) {
		n = conjunto_cardinalidad(c);
		for ( i = 0 ; i < n ; i++ ) {
			e = conjunto_obtenerElemento(c, i);
			printf("%d ", e);
		}
		printf("\n\n");
	}
}

Conjunto *conjunto_union(Conjunto *a, Conjunto *b) {
	Conjunto *r = NULL;
	int i, n, e;

	if ( a == NULL || b == NULL ) {
		return NULL;
	}

	r = conjunto_crear();
	if ( r == NULL ) {
		return NULL;
	}

	n = conjunto_cardinalidad(a);
	for ( i = 0 ; i < n ; i++ ) {
		e = conjunto_obtenerElemento(a, i);
		conjunto_insertarElemento(r, e);
	}

	n = conjunto_cardinalidad(b);
	for ( i = 0 ; i < n ; i++ ) {
		e = conjunto_obtenerElemento(b, i);
		if ( conjunto_buscarElemento(a, e) == -1 ) {
			conjunto_insertarElemento(r, e);
		}
	}

	return r;
}

Conjunto *conjunto_interseccion(Conjunto *a, Conjunto *b) {
	Conjunto *r = NULL;
	int i, n, e;

	if ( a == NULL || b == NULL ) {
		return NULL;
	}

	r = conjunto_crear();
	if ( r == NULL ) {
		return NULL;
	}

	n = conjunto_cardinalidad(a);
	for ( i=0; i<n; i++ ) {
		e = conjunto_obtenerElemento(a, i);
		if ( conjunto_buscarElemento(b, e) != -1 ) {
			conjunto_insertarElemento(r, e);
		}
	}

	return r;
}
