/* =============================================================================
 * elemento.c
 *
 * Interfaz pblica del TAD Elemento - encapsulando un dato de tipo int.
 *
 * Autor: Ivn Cantador
 * Fecha: 9/2/2017
 * Versin: 1.0
 * Comentarios: En este fichero habra que incluir las cabeceras de funcin,
 * describiendo las primitivas, sus argumentos de entrada y valores de salida.
 * ========================================================================== */

#include <stdio.h>
#include <stdlib.h>
#include "tipos.h"
#include "elemento.h"

/* -------------------------- Estructura de datos -------------------------- */
/* ------------------------------------------------------------------------- */

struct _Elemento {
	/* El campo info es int */
	/* En caso de que se cambie el tipo de dato, habra que modifcar el cdigo de algunas primitivas de Elemento */
	int info;
};

/* -------------------------- Funciones primitivas -------------------------- */
/* -------------------------------------------------------------------------- */

Elemento *elemento_crear(int dato) {
	Elemento *e = NULL;

	e = (Elemento *) malloc(sizeof(Elemento));
	if ( e == NULL ) {
		return NULL;
	}

	e->info = dato;

	return e;
}

status elemento_establecerDato(Elemento *elem, int dato) {
	if ( elem == NULL ) {
		return ERROR;
	}

	elem->info = dato;

	return OK;
}

int elemento_obtenerDato(Elemento *elem) {
	if ( elem == NULL ) {
		return NAN;
	}

	return elem->info;
}

Elemento *elemento_copiar(Elemento *elem) {
	Elemento *e = NULL;

	if ( elem == NULL ) {
		return NULL;
	}

	e = (Elemento *) malloc(sizeof(Elemento));
	if ( e == NULL ) {
		return NULL;
	}

	e->info = elem->info;

	return e;
}

int elemento_comparar(Elemento *elem1, Elemento *elem2) {
	if ( elem1 == NULL || elem2 == NULL ) {
		return NAN;
	}

	return elem1->info - elem2->info;
}

void elemento_eliminar(Elemento *elem) {
	if ( elem != NULL ) {
		free(elem);
	}
}

void elemento_visualizar(Elemento *elem) {
	if ( elem != NULL ) {
 		printf("%d\n", elem->info);
	}
}
