/* =============================================================================
 * main3a.c
 *
 * Programa para probar primitivas del TAD Conjunto - Versin 3.
 * Los elementos de un conjunto son void *.
 * En la funcin main se usan conjuntos cuyos elementos son int.
 * Para ello, la primitiva conjunto3_crear recibe punteros a funciones de copia,
 * comparacin, eliminacin y visualizacin de int (definidas abajo).
 *
 * Autor: Ivn Cantador
 * Fecha: 9/2/2017
 * Versin: 1.0
 * ========================================================================== */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tipos.h"
#include "conjunto3.h"

/* Funciones de copia, comparacin, eliminacin y visualizacin de int (ver abajo) */
void *int_copiar(void *elem);
int int_comparar(void *elem1, void *elem2);
void int_eliminar(void *elem);
void int_visualizar(void *elem);

int main3a() {
	Conjunto3 *A = NULL, *B = NULL, *C = NULL;
	int e;

	A = conjunto3_crear(int_copiar, int_comparar, int_eliminar, int_visualizar);
	if ( A == NULL ) {
		return -1;
	}

	B = conjunto3_crear(int_copiar, int_comparar, int_eliminar, int_visualizar);
	if ( B == NULL ) {
		conjunto3_eliminar(A);
		return -1;
	}

	printf("Prueba del TAD Conjunto, version 3 - elementos de tipo void* (almacenando int)\n");

	e = 1; conjunto3_insertarElemento(A, (void *) &e);
	e = 2; conjunto3_insertarElemento(A, (void *) &e);
	e = 3; conjunto3_insertarElemento(A, (void *) &e);
	printf("Conjunto A:\n");
	conjunto3_visualizar(A);

	e = 3; conjunto3_insertarElemento(B, (void *) &e);
	e = 4; conjunto3_insertarElemento(B, (void *) &e);
	e = 5; conjunto3_insertarElemento(B, (void *) &e);
	e = 6; 	conjunto3_insertarElemento(B, (void *) &e);
	printf("Conjunto B:\n");
	conjunto3_visualizar(B);

	C = conjunto3_union(A, B);
	printf("Conjunto Union:\n");
	conjunto3_visualizar(C);
	conjunto3_eliminar(C);

	C = conjunto3_interseccion(A, B);
	printf("Conjunto Interseccion:\n");
	conjunto3_visualizar(C);
	conjunto3_eliminar(C);

	conjunto3_eliminar(A);
	conjunto3_eliminar(B);

	return 1;
}

void *int_copiar(void *elem) {
	int *e = NULL;

	if ( elem == NULL ) {
		return NULL;
	}

	e = (int *) malloc(sizeof(int));
	if ( e == NULL ) {
		return NULL;
	}

	*e = *((int *) elem);

	return e;
}

int int_comparar(void *elem1, void *elem2) {
	int e1, e2;

	if ( elem1 == NULL || elem2 == NULL ) {
		return NAN;
	}

	e1 = *((int *) elem1);
	e2 = *((int *) elem2);

	return e1 - e2;
}

void int_eliminar(void *elem) {
	if ( elem != NULL ) {
		free(elem);
	}
}

void int_visualizar(void *elem) {
	int e;

	if ( elem != NULL ) {
		e = *((int *) elem);
 		printf("%d\n", e);
	}
}
