/* =============================================================================
 * main3b.c
 *
 * Programa para probar primitivas del TAD Conjunto - Versin 3.
 * Los elementos de un conjunto son void *.
 * En la funcin main se usan conjuntos cuyos elementos son char *.
 * Para ello, la primitiva conjunto3_crear recibe punteros a funciones de copia,
 * comparacin, eliminacin y visualizacin de char * (definidas abajo).
 *
 * Autor: Ivn Cantador
 * Fecha: 9/2/2017
 * Versin: 1.0
 * ========================================================================== */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tipos.h"
#include "conjunto3.h"

/* Funciones de copia, comparacin, eliminacin y visualizacin de char * (ver abajo) */
void *string_copiar(void *elem);
int string_comparar(void *elem1, void *elem2);
void string_eliminar(void *elem);
void string_visualizar(void *elem);

int main3b() {
	Conjunto3 *A = NULL, *B = NULL, *C = NULL;

	A = conjunto3_crear(string_copiar, string_comparar, string_eliminar, string_visualizar);
	if ( A == NULL ) {
		return -1;
	}

	B = conjunto3_crear(string_copiar, string_comparar, string_eliminar, string_visualizar);
	if ( B == NULL ) {
		conjunto3_eliminar(A);
		return -1;
	}

	printf("Prueba del TAD Conjunto, version 3 - elementos de tipo void* (almacenando char*)\n");

	conjunto3_insertarElemento(A, (void *) "aaa");
	conjunto3_insertarElemento(A, (void *) "bbb");
	conjunto3_insertarElemento(A, (void *) "ccc");
	printf("Conjunto A:\n");
	conjunto3_visualizar(A);

	conjunto3_insertarElemento(B, (void *) "ccc");
	conjunto3_insertarElemento(B, (void *) "ddd");
	conjunto3_insertarElemento(B, (void *) "eee");
	conjunto3_insertarElemento(B, (void *) "fff");
	printf("Conjunto B:\n");
	conjunto3_visualizar(B);

	C = conjunto3_union(A, B);
	printf("Conjunto Union:\n");
	conjunto3_visualizar(C);
	conjunto3_eliminar(C);

	C = conjunto3_interseccion(A, B);
	printf("Conjunto Interseccion:\n");
	conjunto3_visualizar(C);
	conjunto3_eliminar(C);

	conjunto3_eliminar(A);
	conjunto3_eliminar(B);

	return 1;
}

void *string_copiar(void *elem) {
	char *e = NULL;

	if ( elem == NULL ) {
		return NULL;
	}

	e = (char *) malloc((1 + strlen((char *) elem)) * sizeof(char));
	if ( e == NULL ) {
		return NULL;
	}

	strcpy(e, (char *) elem);

	return e;
}

int string_comparar(void *elem1, void *elem2) {
	char *e1 = NULL, *e2 = NULL;

	if ( elem1 == NULL || elem2 == NULL ) {
		return NAN;
	}

	e1 = (char *) elem1;
	e2 = (char *) elem2;

	return strcmp(e1, e2);
}

void string_eliminar(void *elem) {
	if ( elem != NULL ) {
		free(elem);
	}
}

void string_visualizar(void *elem) {
	char *e = NULL;

	if ( elem != NULL ) {
		e = (char *) elem;
 		printf("%s\n", e);
	}
}
